module state_ctrl(
	clk					,
	rst_n					,
	ddr2_init_done		,
	adc_ch_sel			,
	start_sample		,
	set_sample_num		,
	rdfifo_empty		,
	rdfifo_dout			,
	wrfifo_full			,
	adc_data_flag		,
	rdfifo_clr			,
	rdfifo_rden			,
	wrfifo_clr			,
	ad_sample_en		,
	eth_fifo_wrreq		,
	eth_fifo_wrdata	
);

	input clk;
	input rst_n;
	input ddr2_init_done;
	input [7:0] adc_ch_sel;
	input start_sample;
	input [31:0]set_sample_num;
	input rdfifo_empty;
	input [15:0]rdfifo_dout;
	input wrfifo_full;
	input [7:0] adc_data_flag;
	
	output reg rdfifo_clr;
	output reg wrfifo_clr;
	output reg rdfifo_rden;
	output reg ad_sample_en;
	output reg eth_fifo_wrreq;
	output reg [15:0] eth_fifo_wrdata;
	
	reg [5:0]state;
	//统计ADC向DDR传送的数据，和set_sample_num位深相同
	reg [31:0]adc_sample_cnt;
	//统计向网口发送ADC数据的个数，和set_sample_num位深一致
	reg [31:0]send_data_cnt;
	//将start_sample_rm进行采样，锁定其只在IDLE状态进行工作有效，其余状态均无效
	reg start_sample_rm;
	reg [4:0]wrfifo_clr_cnt;
	reg [4:0]rdfifo_clr_cnt;
	reg [31:0]data_num;
	
   localparam IDLE                   = 6'b000001;   //空闲状态
   localparam DDR_WR_FIFO_CLEAR      = 6'b000010;   //DDR写FIFO清除状态
   localparam ADC_SAMPLE             = 6'b000100;   //ADC采样数据状态
   localparam DDR_RD_FIFO_CLEAR      = 6'b001000;   //DDR读FIFO清除状态
   localparam DATA_SEND_START        = 6'b010000;   //数据发送状态
   localparam DATA_SEND_WORKING      = 6'b100000;   //数据发送完成状态
	
	always  @(posedge clk or negedge rst_n)
	if (!rst_n)
		begin
			state <= IDLE;
			rdfifo_rden <= 1'b0;
			data_num <= 32'd0;
		end
	else 
		case (state)
			IDLE 	: 	begin
							if (start_sample_rm)
								state <= DDR_WR_FIFO_CLEAR;
							else
								state <= state;
						end
						
			DDR_WR_FIFO_CLEAR	:	begin
											if ((!wrfifo_full) && (wrfifo_clr_cnt==9))
												state <= ADC_SAMPLE;
											else
												state <= state;
										end
										
			ADC_SAMPLE	:	begin
									if (adc_sample_cnt >= set_sample_num)
										state <= DDR_RD_FIFO_CLEAR;
									else
										state <= state;
								end
								
			DDR_RD_FIFO_CLEAR	:	begin
											if ((!rdfifo_empty) && (rdfifo_clr_cnt==9))
												state <= DATA_SEND_START;
											else
												state <= state;
										end
										
			DATA_SEND_START	:	begin
												state <= DATA_SEND_WORKING;
										end
										
			DATA_SEND_WORKING	:	begin
											if (send_data_cnt >= set_sample_num - 1'b1)
												begin
													rdfifo_rden <= 1'b0;
													state <= IDLE;
												end
											else
												begin
													state <= DATA_SEND_WORKING;
													rdfifo_rden <= 1'b1;
												end
										end	
		endcase
		
		
	issp 	issp(
	.probe({{state},{eth_fifo_wrreq},{eth_fifo_wrdata}}),
	.source()
	); 

		
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)
		begin
			eth_fifo_wrreq <= 1'b0;
			eth_fifo_wrdata <= 16'd0;
		end
	else if (rdfifo_rden)
		begin
			eth_fifo_wrreq <= 1'b1;
			eth_fifo_wrdata <= rdfifo_dout; 
		end
	else
		begin
			eth_fifo_wrreq <= 1'b0;
			eth_fifo_wrdata <= 16'd0;
		end
		
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)
		start_sample_rm <= 1'b0;
	else if (state == IDLE && ddr2_init_done == 1)
		start_sample_rm <= start_sample;
	else
		start_sample_rm <= 1'b0;
		
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)
		wrfifo_clr_cnt <= 'd0;
	else if (state == DDR_WR_FIFO_CLEAR)
		begin
			if (wrfifo_clr_cnt == 'd9)
				wrfifo_clr_cnt <= 'd9;
			else
				wrfifo_clr_cnt <= wrfifo_clr_cnt+ 1'd1;
		end
	else
		wrfifo_clr_cnt <= 'd0;

//*//初始化成功后，进行一次清fifo
//*如果进入了DDR_WR_FIFO_CLEAR状态，则在wrfifo_clr_cnt为0,1或2时，清写fifo置1，否则wrfifo_clr为0*//
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)
		wrfifo_clr<=0;
	else if (ddr2_init_done == 0)
		wrfifo_clr<=1'b1;
	else if(state==DDR_WR_FIFO_CLEAR)
        begin
            if(wrfifo_clr_cnt==0||wrfifo_clr_cnt==1||wrfifo_clr_cnt==2)
                wrfifo_clr<=1'b1;
            else
                wrfifo_clr<=1'b0;
        end
	else 
		wrfifo_clr<=1'b0;
		
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)
		ad_sample_en<=0;
    else if(state==ADC_SAMPLE)
		ad_sample_en<=1;
    else
		ad_sample_en<=0;

//以下//如果adc_sample_cnt在ADC_SAMPLE状态，则每个时钟周期自加1
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)                                 
		adc_sample_cnt<=32'd0;
    else if(state==ADC_SAMPLE) begin
		if(adc_ch_sel & adc_data_flag)
			adc_sample_cnt<=adc_sample_cnt+1'b1;
		else
			adc_sample_cnt<=adc_sample_cnt;
		end
    else
		adc_sample_cnt<=32'd0;

//以下//清除DDR读FIFO的计数器，10拍后指挥状态的跳转
	always @ (posedge clk or negedge rst_n)
	if (!rst_n)  
		rdfifo_clr_cnt<= 'd0;
	else if(state == DDR_RD_FIFO_CLEAR)//如果进入了清fifo状态
		begin 
			if(rdfifo_clr_cnt == 9)
				rdfifo_clr_cnt <= 'd9;
			else
				rdfifo_clr_cnt <= rdfifo_clr_cnt+1'b1;
		end	
	else
		rdfifo_clr_cnt<= 'd0;

	always @ (posedge clk or negedge rst_n)
	if (!rst_n)  
		rdfifo_clr<=0;
	else if (ddr2_init_done == 0)
		rdfifo_clr<= 1;
   else if(state==DDR_RD_FIFO_CLEAR)
		begin
			if(rdfifo_clr_cnt==0||rdfifo_clr_cnt==1||rdfifo_clr_cnt==2)
				rdfifo_clr<=1'b1;
			else
				rdfifo_clr<=1'b0;
		end
	else 
		rdfifo_clr<=1'b0;

/*每个send_data_cnt在rdfifo_rden为1的状态下加1，
由于rdfifo_rden为高每次只持续一拍，
保证了每次读16bit数时send_data_cnt只持续加1*/
	always @ (posedge clk or negedge rst_n)
	if (!rst_n) 
		send_data_cnt<=32'd0;
   else if(state==IDLE)
		send_data_cnt<=32'd0;
   else if(rdfifo_rden)
		send_data_cnt<=send_data_cnt+1;
   else 
		send_data_cnt<=send_data_cnt;


endmodule 

